package com.flywithu.carnav;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

import com.hoho.android.usbserial.driver.UsbSerialDriver;
import com.hoho.android.usbserial.driver.UsbSerialPort;
import com.hoho.android.usbserial.driver.UsbSerialProber;

import java.util.List;

public class MainActivity extends AppCompatActivity {

    public static final String SERVICECMD = "com.android.music.musicservicecommand";
    public static final String TAG = "carnav";
    static final String ACTION_USB_PERMISSION = "com.flywithu.carnav.USB_PERMISSION";
    private Intent serviceIntent;
    private BroadcastReceiver usbReceiver = null;

    private void goService() {


        if (RealService.serviceIntent == null) {

            serviceIntent = new Intent(this, RealService.class);
            startService(serviceIntent);
            Toast.makeText(getApplicationContext(), "new start", Toast.LENGTH_LONG).show();
        } else {
            serviceIntent = RealService.serviceIntent;//getInstance().getApplication();
            Toast.makeText(getApplicationContext(), "already", Toast.LENGTH_LONG).show();
        }

    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);


        UsbManager manager = (UsbManager) getSystemService(Context.USB_SERVICE);
        List<UsbSerialDriver> availableDrivers = UsbSerialProber.getDefaultProber().findAllDrivers(manager);
        if (!availableDrivers.isEmpty()) {
            Toast.makeText(getApplicationContext(), "USB Exist", Toast.LENGTH_LONG).show();

            UsbSerialDriver driver = null;
            UsbDeviceConnection connection = null;
            try {
                driver = availableDrivers.get(0);
                if (!manager.hasPermission(driver.getDevice())) {

                    Toast.makeText(getApplicationContext(), "NO Permission", Toast.LENGTH_LONG).show();
                    //Request USB HOST
                    // You probably need to call UsbManager.requestPermission(driver.getDevice(), ..)
                    if (!manager.hasPermission(driver.getDevice())) {

                        PendingIntent mPermissionIntent;
                        mPermissionIntent = PendingIntent.getBroadcast(MainActivity.this, 0,
                                new Intent(ACTION_USB_PERMISSION), PendingIntent.FLAG_ONE_SHOT);
//

                        usbReceiver = new BroadcastReceiver() {
                            @Override
                            public void onReceive(Context context, Intent intent) {
                                String action = intent.getAction();
                                if (ACTION_USB_PERMISSION.equals(action)) {

                                    if (intent.getBooleanExtra(UsbManager.EXTRA_PERMISSION_GRANTED, true)) {
                                        Toast.makeText(getApplicationContext(), "Pemission Granted", Toast.LENGTH_LONG).show();
                                        goService();
                                    } else {
                                        Toast.makeText(getApplicationContext(), "Pemission Denied", Toast.LENGTH_LONG).show();
                                    }
                                }

                            }
                        };

                        registerReceiver(usbReceiver, new IntentFilter(ACTION_USB_PERMISSION));
                        manager.requestPermission(driver.getDevice(), mPermissionIntent);


                    }
                } else {
                    Toast.makeText(getApplicationContext(), "Has Permission", Toast.LENGTH_LONG).show();
                    connection = manager.openDevice(driver.getDevice());
                    if (connection == null) {
                        Toast.makeText(getApplicationContext(), "Unknown error", Toast.LENGTH_LONG).show();
                    } else {
                        UsbSerialPort sPort = null;
                        sPort = driver.getPorts().get(0);
                        sPort.open(connection);
                        Toast.makeText(getApplicationContext(), "Serial Count:" + String.valueOf(driver.getPorts().size()), Toast.LENGTH_LONG).show();

                        sPort.close();
                        sPort = null;


                        goService();
                    }
                }
            } catch (Exception e) {
                Toast.makeText(getApplicationContext(), e.toString(), Toast.LENGTH_LONG).show();
            }
        } else {
            Toast.makeText(getApplicationContext(), "NO USB", Toast.LENGTH_LONG).show();
        }
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (serviceIntent != null) {
            stopService(serviceIntent);
            serviceIntent = null;
        }
        if (usbReceiver != null) {
            unregisterReceiver(usbReceiver);
        }
    }

}
