package com.flywithu.carnav;

import android.app.AlarmManager;
import android.app.Application;
import android.app.PendingIntent;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

import com.hoho.android.usbserial.driver.UsbSerialDriver;
import com.hoho.android.usbserial.driver.UsbSerialPort;
import com.hoho.android.usbserial.driver.UsbSerialProber;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class RealService extends Service {
    //   private Thread mainThread;
    public static Intent serviceIntent = null;

    private static UsbSerialPort sPort = null;
    private MusicTitle currentSong = new MusicTitle();
    private BroadcastReceiver mReceiver = new BroadcastReceiver() {

        @Override
        public void onReceive(Context context, Intent intent) {


            Bundle bundle = intent.getExtras();
            if (bundle != null) {
                for (String key : bundle.keySet()) {
                    Object value = bundle.get(key);
                    Log.d("MUSIC", String.format("%s %s (%s)", key,
                            value.toString(), value.getClass().getName()));
                }
            }

            String action = intent.getAction();
            String cmd = intent.getStringExtra("command");
            Log.v("MUSIC ", action + " / " + cmd);
            String artist = intent.getStringExtra("artist");
            String album = intent.getStringExtra("album");
            String track = intent.getStringExtra("track");
            Long myPosition = intent.getLongExtra("id", 1);
            Log.v("MUSIC", artist + ":" + album + ":" + track);
            boolean isRunning = intent.getBooleanExtra("playstate", false) | intent.getBooleanExtra("playing", false);

            currentSong.setRunning(isRunning);

            if (isRunning == true) {
                currentSong.setAlbum(album);
                currentSong.setArtist(artist);
                currentSong.setTrack(track);
                currentSong.setPosition(myPosition.intValue());
            }

        }
    };

    public RealService() {
    }

    private void sendSerial(String str) {

        sendSerial(str, Charset.forName("UTF-8"));
    }

    private synchronized void sendSerial(String str, Charset enc) {

        if (sPort != null) {
            try {

                int len = sPort.write(getStringToByte(str, enc), 3000);
                sPort.write(new byte[]{0xd}, 3000);
                Log.d("TAG", str);
            } catch (Exception e) {
                showToast(getApplication(), "Error:" + e.toString());
            }

        } else {
            showToast(getApplication(), "SERIAL NULL");
        }


    }

    protected byte[] getStringToByte(String str, Charset enc) {
        ByteBuffer buf = ByteBuffer.allocate(150);

        String[] arrStr = null;
        if (str.startsWith("U4PL13") || str.startsWith("U4PL23") || str.startsWith("U4PL33")) {
            arrStr = str.split("\\|");
            buf.put(arrStr[0].getBytes(Charset.forName("UTF-8")));
            buf.put(new byte[]{0x7c});
            buf.put(arrStr[1].getBytes(enc));
            buf.put(new byte[]{0x7c});

        } else {
            buf.put(str.getBytes(Charset.forName("UTF-8")));
        }


        byte[] bytes = new byte[buf.position()];
        buf.flip();
        buf.get(bytes);


        return bytes;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        serviceIntent = intent;
        showToast(getApplication(), "Start Service");

        UsbManager manager = (UsbManager) getSystemService(Context.USB_SERVICE);
        List<UsbSerialDriver> availableDrivers = UsbSerialProber.getDefaultProber().findAllDrivers(manager);
        if (availableDrivers.isEmpty()) {
            showToast(getApplication(), "NO SERIAL");
            return START_NOT_STICKY;
        }

        UsbSerialDriver driver = null;
        UsbDeviceConnection connection = null;
        try {
            driver = availableDrivers.get(0);

            connection = manager.openDevice(driver.getDevice());
            if (connection == null) {
                showToast(getApplication(), "No connection");
                return START_NOT_STICKY;

            }
            sPort = driver.getPorts().get(0);


        } catch (Exception e) {
            showToast(getApplication(), e.toString());
            return START_NOT_STICKY;
        }

        try {

            sPort.open(connection);
            sPort.setParameters(19200, 8, UsbSerialPort.STOPBITS_1, UsbSerialPort.PARITY_NONE);


        } catch (IOException e) {
            showToast(getApplication(), "Error opening device: " + e.getMessage());
            try {
                sPort.close();
            } catch (IOException e2) {
                // Ignore.
            }
            sPort = null;
            return START_NOT_STICKY;
        }


        SimpleDateFormat format1 = new SimpleDateFormat("00yyMMddHHmm");
        //   String f = format1.format(System.currentTimeMillis());
        String currentInfo = format1.format(new Date());
        String currentTime = String.format("TMST%s", currentInfo);
        sendSerial(currentTime);
        sendSerial("U4EN00");

        IntentFilter iF = new IntentFilter();
        iF.addAction("com.android.music.metachanged");
        iF.addAction("com.htc.music.metachanged");
        iF.addAction("fm.last.android.metachanged");
        iF.addAction("com.sec.android.app.music.metachanged");
        iF.addAction("com.nullsoft.winamp.metachanged");
        iF.addAction("com.amazon.mp3.metachanged");
        iF.addAction("com.miui.player.metachanged");
        iF.addAction("com.real.IMP.metachanged");
        iF.addAction("com.sonyericsson.music.metachanged");
        iF.addAction("com.rdio.android.metachanged");
        iF.addAction("com.samsung.sec.android.MusicPlayer.metachanged");
        iF.addAction("com.andrew.apollo.metachanged");
        iF.addAction("com.android.music.metachanged");
        iF.addAction("com.android.music.playstatechanged");
        iF.addAction("com.android.music.playbackcomplete");
        iF.addAction("com.android.music.queuechanged");


        registerReceiver(mReceiver, iF);


        new Thread(new Runnable() {
            @Override
            public void run() {

                boolean myRun = true;
                int stopCount = 0;
                while (myRun) {
                    try {


                        Thread.sleep(5000);
                        SimpleDateFormat format1 = new SimpleDateFormat("00yyMMddHHmm");
                        String currentInfo = format1.format(new Date());
                        String currentTime = String.format("TMST%s", currentInfo);
                        sendSerial(currentTime);

                        if (currentSong.isRunning()) {
                            stopCount = 0;
                            sendSerial("U4EN00");
                            sendSerial("U4OP00");
                            sendSerial("U4PL03|" + currentSong.getTotal() + "," + currentSong.getPosition() + "|0001010000000");
                            sendSerial("U4PL13|" + currentSong.getAlbum() + "|", Charset.forName("UTF-8"));
                            sendSerial("U4PL23|" + currentSong.getTrack() + "|", Charset.forName("UTF-8"));
                            sendSerial("U4PL33|" + currentSong.getArtist() + "|", Charset.forName("UTF-8"));
                        } else {
                            if (stopCount > 5) {
                                sendSerial("U4DI00");
                            }
                            stopCount++;
                        }


                    } catch (InterruptedException e) {
                        myRun = false;

                    }
                }

            }

        }).start();


        return START_NOT_STICKY;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        try {
            sPort.close();
            sPort = null;
        } catch (Exception e) {
            showToast(getApplication(), "stop Error");
        }
        serviceIntent = null;
        setAlarmTimer();
        Thread.currentThread().interrupt();
        unregisterReceiver(mReceiver);
        showToast(getApplication(), "stop Service");

    }

    @Override
    public void onCreate() {
        super.onCreate();
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        return super.onUnbind(intent);
    }

    public void showToast(final Application application, final String msg) {
        Handler h = new Handler(application.getMainLooper());
        h.post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(application, msg, Toast.LENGTH_SHORT).show();
            }
        });
    }

    protected void setAlarmTimer() {
        final Calendar c = Calendar.getInstance();
        c.setTimeInMillis(System.currentTimeMillis());
        c.add(Calendar.SECOND, 1);
        Intent intent = new Intent(this, AlarmReceiver.class);
        PendingIntent sender = PendingIntent.getBroadcast(this, 0, intent, 0);

        AlarmManager mAlarmManager = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
        mAlarmManager.set(AlarmManager.RTC_WAKEUP, c.getTimeInMillis(), sender);
    }


}